/*
 * Copyright (c) 2016  Marien Raat <marienraat@riseup.net>
 * Copyright (c) 2017  Stephen Michel <s@smichel.me>
 *
 *  This file is free software: you may copy, redistribute and/or modify it
 *  under the terms of the GNU General Public License as published by the Free
 *  Software Foundation, either version 3 of the License, or (at your option)
 *  any later version.
 *
 *  This file is distributed in the hope that it will be useful, but WITHOUT ANY
 *  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 *  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 *  details.
 *
 *  You should have received a copy of the GNU General Public License along with
 *  this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * This file incorporates work covered by the following copyright and
 * permission notice:
 *
 *     Copyright (c) 2015 Chris Nguyen
 *     Copyright (c) 2016 Zoraver <https://github.com/Zoraver>
 *
 *     Permission to use, copy, modify, and/or distribute this software
 *     for any purpose with or without fee is hereby granted, provided
 *     that the above copyright notice and this permission notice appear
 *     in all copies.
 *
 *     THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL
 *     WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED
 *     WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE
 *     AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR
 *     CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS
 *     OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 *     NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
 *     CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */
package com.jmstudios.redmoon.manager

import android.content.Context
import android.provider.Settings
import com.jmstudios.redmoon.model.Config
import com.jmstudios.redmoon.helper.Logger
import com.jmstudios.redmoon.helper.Permission

class BrightnessManager(private val mContext: Context) {
    companion object: Logger()

    fun lower() =  when {
        !Config.filterIsOn       -> Log.w("Can't lower brightness; filter is off!")
        Config.brightnessLowered -> Log.w("Brightness is already lowered!")
        !Config.lowerBrightness  -> Log.w("Lower brightness not enabled!")
        !Permission.WriteSettings.isGranted -> Log.w("Permission not granted!")
        else -> try {
            val resolver = mContext.contentResolver
            val oldLevel = Settings.System.getInt(resolver, Settings.System.SCREEN_BRIGHTNESS)
            val oldAuto = 1 == Settings.System.getInt(resolver, "screen_brightness_mode")
            Config.automaticBrightness = oldAuto
            Config.brightness = oldLevel

            Log.i("Lowering brightness from: $oldLevel, auto: $oldAuto")
            Settings.System.putInt(resolver, Settings.System.SCREEN_BRIGHTNESS, 0)
            Settings.System.putInt(resolver, "screen_brightness_mode", 0)
            Config.brightnessLowered = true
        } catch (e: Settings.SettingNotFoundException) {
            Log.e("Error reading brightness state $e")
        }
    }

    fun restore() = when {
        !Config.brightnessLowered -> Log.w("Can't restore brightness; it's not lowered!")
        !Permission.WriteSettings.isGranted -> Log.w("Permission not granted!")
        else -> {
            val resolver = mContext.contentResolver
            val automatic = if (Config.automaticBrightness) 1 else 0

            Log.i("Restoring brightness to: ${Config.brightness}, automatic: $automatic")
            Settings.System.putInt(resolver, "screen_brightness_mode", automatic)
            Settings.System.putInt(resolver, Settings.System.SCREEN_BRIGHTNESS, Config.brightness)
            Config.brightnessLowered = false
        }
    }
}
